
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <limits.h>
#include <fx.h>

static int app_created;

#include "gui.h"


#include "actions.h"
#include "help.h"



 typedef struct radio_item {
   struct radio_item *next;
   FXRadioButton *button;
   int value;
 } radio_item_t;

 typedef struct {
   int *target;
   radio_item_t *head;
 } FYRadioGroup;


  
static void set_value
  (FYRadioGroup *G, int V)
{
  radio_item_t *T;
  for(T=G->head;T;T=T->next)
    if (T->value==V) 
     T->button->setCheck(TRUE);
    else T->button->setCheck(FALSE);
  G->target[0]= V;
}

static void get_value
  (FYRadioGroup *G,int *R)
{
  *R= G->target[0];
}

static void append(FYRadioGroup *G, FXRadioButton *B, int V)
{
  radio_item_t *T= (radio_item_t*) malloc(sizeof(*T));
  T->button= B;
  T->value= V;
  T->next= G->head;
  G->head= T;
}


static void radio_group_set_value
  (gui_radio_group_t *G, int V)
{
  set_value( (FYRadioGroup*) (G->widget), V);
}

static void radio_group_get_value
  (gui_radio_group_t *G,int *R)
{
  get_value( (FYRadioGroup*) (G->widget), R);
}

static gui_radio_group_class_t
gui_radio_group_class = {
  radio_group_set_value,
  radio_group_get_value,
};

  
static void label_set_text(gui_label_t *B, char *text)
{
  FXLabel *W= (FXLabel*) B->widget;
  W->setText(text);
}

static gui_label_class_t
 gui_label_class= {
  label_set_text,
};


    static FXFont *YQRZ_8;
       
class FYListItem : public FXListItem 
{
  FXDECLARE(FYListItem)
protected:
  FYListItem() : FXListItem() { }
public:
  FYListItem(const FXString& text,FXIcon* ic, void *pdata)
   : FXListItem(text,ic,NULL)
  { data_p= pdata; }
  FYListItem(const FXString& text,FXIcon* ic, int idata)
   : FXListItem(text,ic,NULL)
  { data_i= idata; }
  FYListItem(const FXString& text,FXIcon* ic, double fdata)
   : FXListItem(text,ic,NULL)
  { data_f= fdata; }
  ~FYListItem() { }
// These could be in a union, but who cares about space
  int data_i;
  double data_f;
  void *data_p;
};


FXIMPLEMENT(FYListItem,FXListItem,NULL,0)


   
static void clear_selection
  (FXList *L)
{
  int i;
  int N= L->getNumItems();
  for(i=0;i<N;i++)
    L->deselectItem(i);
}

static FYListItem* get_selected(FXList *L)
{
   int i, N= L->getNumItems();
   for(i=0;i<N;i++)
     if (L->isItemSelected(i))
       return (FYListItem*) L->getItem(i);
   return 0;
}


   
static FXList* list(gui_slist_t *SL)
{
  return (FXList*) SL->widget;
}

static int slist_get_selected(gui_slist_t *SL, char **ret)
{
  FYListItem *item= get_selected(list(SL));
  if (!item) return 0;
  *ret= (char*) item->data_p;
  return 1;
}

static int slist_find_item(gui_slist_t *SL,const char *key)
{
  FXList *L= list(SL);
  int i, N= L->getNumItems();
  for(i=0;i<N;i++) {
    FYListItem* item= (FYListItem*) L->getItem(i);
    if (!strcmp((char*) item->data_p, key)) return i;
  }
  return -1;
}

static void slist_append_item
    (gui_slist_t *SL, const char *value, const char *data)
{
  list(SL)->appendItem(new FYListItem(value, NULL, (void*) data));
}

static void slist_prepend_item
   (gui_slist_t *SL, const char *value,const char *data)
{
  list(SL)->prependItem(new FYListItem(value, NULL, (void*) data));
}

static void slist_insert_item
      (gui_slist_t *SL, const char *poskey,
       const char *value,const char *data)
{
  int i= slist_find_item(SL, poskey);
  if (i==-1) slist_append_item(SL, value,data);
  else list(SL)->insertItem(i,new FYListItem(value, NULL, (void*) data));
}

static void slist_remove_item
      (gui_slist_t *SL, const char *key)
{
  int i= slist_find_item(SL, key);
  if (i!=-1)
     list(SL)->removeItem(i);
}

static gui_slist_class_t gui_slist_class= {
  slist_get_selected,
  slist_append_item,
  slist_prepend_item,
  slist_insert_item,
  slist_remove_item,
};

   
static void button_set_text(gui_button_t *B,const char *text)
{
  FXButton *W= (FXButton*) B->widget;
  W->setText(text);
}

static gui_button_class_t
 gui_button_class= {
  button_set_text,
};


    
static void check_box_get_value
     (gui_check_box_t *CB,int *R)
{
  FXCheckButton *W= (FXCheckButton*) (CB->widget);
  // I know, I`m checking against TRUE but this is necessary
  // since the library has also a MAYBE value. peh.
  if (W->getCheck()==TRUE)
   *R= 1;
  else *R= 0;
}

static void check_box_set_value(gui_check_box_t *CB, int V)
{
  FXCheckButton *W= (FXCheckButton*) (CB->widget);
  if (V) W->setCheck(TRUE); else W->setCheck(FALSE);
}

static gui_check_box_class_t gui_check_box_class= {
  check_box_get_value,
  check_box_set_value,
};

      
static void spinner_set_value(gui_spinner_t *B, int value)
{
  FXSpinner *W= (FXSpinner*) B->widget;
  W->setValue(value);
}

static void spinner_get_value(gui_spinner_t *B, int *ret)
{
  FXSpinner *W= (FXSpinner*) B->widget;
  *ret= W->getValue();
}

static gui_spinner_class_t
 gui_spinner_class= {
  spinner_set_value,
  spinner_get_value,
};


      
static void text_entry_set_text(gui_text_entry_t *B, char *text)
{
  FXTextField *W= (FXTextField*) B->widget;
  W->setText(text);
}

static char* text_entry_get_text(gui_text_entry_t *B)
{
  FXTextField *W= (FXTextField*) B->widget;
  FXString S= W->getText();
  return strdup(S.text());
}

static 
 gui_text_entry_class_t gui_text_entry_class= {
   text_entry_set_text,
   text_entry_get_text
};

       static FXFont *YQRZ_138;
        

static void gui_message_console_print
   (gui_message_console_t *mc, const char * fmt,...)
{
   int size=200; char *message; int len; char *atmp;
   va_list ap;
   message= (char*) malloc(size);
   while(1) {
     va_start(ap,fmt);
     len= vsnprintf(message,size, fmt, ap);
     va_end(ap);
     if (len<0) size*= 2;
     else if (len<size) break;
     else size= len+2;
     message= (char*) realloc(atmp= message, size);
     if (!message) {
        free(atmp);
        message= (char*) fmt;
        break;
     }
  } 
  FXText *W= (FXText*) mc->widget;
  W->appendText(message, strlen(message));
  if (message!=fmt) free(message);
   // argh, AUTOSCROLL doesn`t work yet. see
   // http://ruby.11.n6.nabble.com/TEXT-AUTOSCROLL-in-FXText-with-usage-of-FXDataTarget-td3510118.html
  W->makePositionVisible(W->getLength()-1);
}


static void gui_message_console_clear(gui_message_console_t *mc)
{
  FXText *W= (FXText*) mc->widget;
  W->removeText(0, W->getLength()); 
}

static gui_message_console_class_t 
  gui_message_console_class= 
{
  gui_message_console_print,
  gui_message_console_clear
};


typedef struct {
   int (*execute)(void*);
   void (*close)(void*);
} gui_generic_dialog_class_t;

typedef struct {
  void *cls;
  void *widget;
} gui_ptrpair_t;

static int dialog_execute(void *form)
{
  gui_ptrpair_t *W= (gui_ptrpair_t*) form;
  FXTopWindow *win= (FXTopWindow*) W->widget;
  win->create();
  win->show(PLACEMENT_SCREEN);
  FXApp::instance()->refresh();
  return FXApp::instance()->runModalFor(win);
}

static void dialog_close(void *form)
{
  gui_ptrpair_t *W= (gui_ptrpair_t*) form;
  FXTopWindow *win= (FXTopWindow*) W->widget;
  delete win;
  free(form);
}

static gui_generic_dialog_class_t gui_dialog_class= {
   dialog_execute,
   dialog_close
};

  
static int item_type(FXDirItem *item)
{
  if (item->isDirectory()) return 'D';
  if (item->isSymlink())   return 'L';
  if (item->isChardev())   return 'C';
  if (item->isBlockdev())  return 'B';
  if (item->isFifo())      return 'P';
  if (item->isSocket())    return 'S';
  return 'F';
}

static int dirlist_get_current
  (gui_dir_list_t* DL, const char **Rname, int *Rtype)
{
  FXDirList *L= (FXDirList*) DL->widget;
  FXDirItem *item= (FXDirItem*) L->getCurrentItem();
  if (!item) return 0;
  if (Rname) *Rname= strdup(L->getItemPathname(item).text());
  if (Rtype) *Rtype= item_type(item);
  return 1;
}

static void dirlist_set_current_file
  (gui_dir_list_t *DL, const char *fn)
{
  FXDirList *L= (FXDirList*) DL->widget;
  L->setCurrentFile(fn);
}

static void dirlist_set_current_directory
  (gui_dir_list_t *DL, const char *fn)
{
  FXDirList *L= (FXDirList*) DL->widget;
  L->setDirectory(fn);
}

static gui_dir_list_class_t gui_dir_list_class= {
   dirlist_get_current,
   dirlist_set_current_file,
   dirlist_set_current_directory 
};

  static FXFont *YQRZ_232;
       
static void tv_set_text
  (gui_text_viewer_t *TV, const char *v)
{
  FXText *T= (FXText*) (TV->widget);
  T->setText(v);
}

static void tv_clear
  (gui_text_viewer_t *TV)
{
  FXText *T= (FXText*) (TV->widget);
  T->removeText(0, T->getLength()); 
}

 static gui_text_viewer_class_t gui_text_viewer_class= 
 {
   tv_clear,
   tv_set_text,
 };

  

 class VFLL1: public FXMainWindow
 {
  FXDECLARE(VFLL1)
 public:
  enum { ID_FIRST= FXMainWindow::ID_LAST,
         ID_SELF,
          YQRZ_10,

   YQRZ_15,

   YQRZ_20,

    

         


     YQRZ_41,

      YQRZ_49,

      YQRZ_54,

      YQRZ_59,

        


     YQRZ_68,

      YQRZ_76,

      YQRZ_81,

      YQRZ_86,

        


     YQRZ_95,

         


     YQRZ_108,

     YQRZ_113,

     YQRZ_131,

   YQRZ_136,

   YQRZ_141,

    };
    FYRadioGroup *eof_indicator;
  int eof_indicator_store;
 // FIXME: I should find some prefix and not use the suffix _store..


  FYRadioGroup *alarm;
  int alarm_store;
 // FIXME: I should find some prefix and not use the suffix _store..



  

   FXList *source_dirs;

 
  long YQRZ_13(FXObject *object,FXSelector sel, void *arg);
  

  
  long YQRZ_18(FXObject *object,FXSelector sel, void *arg);
  

  
  long YQRZ_23(FXObject *object,FXSelector sel, void *arg);
  

    FXCheckButton* size_limit_on;
  

    
  FXSpinner *size_limit;

    
  

    
FXTextField *destdir;

    
  long YQRZ_44(FXObject *object,FXSelector sel, void *arg);
  

      long YQRZ_52(FXObject *object,FXSelector sel, void *arg);
 
      long YQRZ_57(FXObject *object,FXSelector sel, void *arg);
 
      long YQRZ_62(FXObject *object,FXSelector sel, void *arg);
 
    
FXTextField *eof_file;

    
  long YQRZ_71(FXObject *object,FXSelector sel, void *arg);
  

      long YQRZ_79(FXObject *object,FXSelector sel, void *arg);
 
      long YQRZ_84(FXObject *object,FXSelector sel, void *arg);
 
      long YQRZ_89(FXObject *object,FXSelector sel, void *arg);
 
    
FXTextField *alarm_file;

    
  long YQRZ_98(FXObject *object,FXSelector sel, void *arg);
  

    
  

    
FXTextField *alarm_player;

    
  long YQRZ_111(FXObject *object,FXSelector sel, void *arg);
  

    
  long YQRZ_116(FXObject *object,FXSelector sel, void *arg);
  

    
  

  
  FXSpinner *gui_check_period;

  
  

  
  long YQRZ_134(FXObject *object,FXSelector sel, void *arg);
  

  
  long YQRZ_139(FXObject *object,FXSelector sel, void *arg);
  

  
  long YQRZ_144(FXObject *object,FXSelector sel, void *arg);
  

  
  FXText *status;

  
  VFLL1(FXApp* a,const FXString& name,
         FXIcon *ic,FXIcon *mi,FXuint opts,
         FXint x,FXint y,FXint w,FXint h,
         FXint pl,FXint pr,FXint pt,FXint pb,
         FXint hs,FXint vs);
  virtual ~VFLL1();
  VFLL1() : FXMainWindow() { }
  long handle_close(FXObject *object,FXSelector sel, void *arg);
 };

 FXDEFMAP(VFLL1) YQRZ_150 []= {
  FXMAPFUNC(SEL_CLOSE, VFLL1::ID_SELF, VFLL1::handle_close),
   
    FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_10, VFLL1::YQRZ_13),

   
    FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_15, VFLL1::YQRZ_18),

   
    FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_20, VFLL1::YQRZ_23),

    

     

     
    FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_41, VFLL1::YQRZ_44),

      FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_49, VFLL1::YQRZ_52),

      FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_54, VFLL1::YQRZ_57),

      FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_59, VFLL1::YQRZ_62),

     

     
    FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_68, VFLL1::YQRZ_71),

      FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_76, VFLL1::YQRZ_79),

      FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_81, VFLL1::YQRZ_84),

      FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_86, VFLL1::YQRZ_89),

     

     
    FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_95, VFLL1::YQRZ_98),

     

     
    FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_108, VFLL1::YQRZ_111),

     
    FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_113, VFLL1::YQRZ_116),

     
    FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_131, VFLL1::YQRZ_134),

   
    FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_136, VFLL1::YQRZ_139),

   
    FXMAPFUNC(SEL_COMMAND, VFLL1::YQRZ_141, VFLL1::YQRZ_144),

   
 }; 

 FXIMPLEMENT(VFLL1, FXMainWindow, YQRZ_150, ARRAYNUMBER(YQRZ_150))

 // FIXME: the argument names must be changed so that
 // we can have widgets called 'y' etc.
 VFLL1::VFLL1(
         FXApp* a,const FXString& name,
         FXIcon *ic,FXIcon *mi,FXuint opts,
         FXint x,FXint y,FXint w,FXint h,
         FXint pl,FXint pr,FXint pt,FXint pb,
         FXint hs,FXint vs) :
               FXMainWindow(a,name, ic,mi,opts,
                            x,y,w,h, pl,pr,pt,pb, hs,vs)

 {
   setTarget(this);
   setSelector(ID_SELF);
   
  FYRadioGroup *YQRZ_1;
  YQRZ_1= eof_indicator = (FYRadioGroup*) malloc(sizeof(* YQRZ_1));
  YQRZ_1->target= & eof_indicator_store;
  YQRZ_1->head= NULL;


  FYRadioGroup *YQRZ_2;
  YQRZ_2= alarm = (FYRadioGroup*) malloc(sizeof(* YQRZ_2));
  YQRZ_2->target= & alarm_store;
  YQRZ_2->head= NULL;

  FXVerticalFrame *YQRZ_149= new
    FXVerticalFrame( this,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        LAYOUT_FILL_Y  ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

 
  FXLabel *YQRZ_3;

  YQRZ_3= 
    new FXLabel(YQRZ_149,
      "Source Directories" ,
      NULL,

        LAYOUT_CENTER_X  |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        0 |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


    FXList *YQRZ_6;

   YQRZ_6= source_dirs = new FXList(YQRZ_149, NULL, 0,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        LAYOUT_FILL_Y  
                 ,0,0,0,0); // xywh


  
     FXFontDesc YQRZ_7;
     YQRZ_6->getFont()->getFontDesc(YQRZ_7);
  

  

  
     strcpy(YQRZ_7.face, "mono");
  

  
    YQRZ_8= new FXFont(FXApp::instance(), YQRZ_7);
    YQRZ_6->setFont(YQRZ_8);
       


   FXHorizontalFrame *YQRZ_24= new
    FXHorizontalFrame( YQRZ_149,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

  
  FXButton *YQRZ_9;

  YQRZ_9= 
    new FXButton(YQRZ_24,
      "Add" ,
      NULL,
      this ,
      YQRZ_10 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

  
  FXButton *YQRZ_14;

  YQRZ_14= 
    new FXButton(YQRZ_24,
      "Remove" ,
      NULL,
      this ,
      YQRZ_15 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

  
  FXButton *YQRZ_19;

  YQRZ_19= 
    new FXButton(YQRZ_24,
      "Scan" ,
      NULL,
      this ,
      YQRZ_20 ,

        LAYOUT_RIGHT  |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

    FXGroupBox *YQRZ_47;
  YQRZ_47= new FXGroupBox(YQRZ_149, "Destination Directory" , 
        GROUPBOX_TITLE_LEFT |  // FIXME: this shall be configurable
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 |
        FRAME_LINE |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,
     0,0,0,0, // xywh
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

    FXVerticalFrame *YQRZ_46= new
    FXVerticalFrame( YQRZ_47,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        LAYOUT_FILL_Y  ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

     FXHorizontalFrame *YQRZ_36= new
    FXHorizontalFrame( YQRZ_46,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

      FXCheckButton *YQRZ_25;
  YQRZ_25= size_limit_on = new FXCheckButton(
             YQRZ_36, "Limit size of copied files to: " ,
                     NULL,
              0,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_NONE |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

             0,0,0,0, // xywh
             DEFAULT_PAD,
             DEFAULT_PAD,
             DEFAULT_PAD,
             DEFAULT_PAD);

  

  

  

  


    
  FXSpinner *YQRZ_30;

  YQRZ_30= size_limit =
    new FXSpinner(YQRZ_36, 5 ,
       // target and selector
       NULL, 0,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |
        0 |
       // there are some options, but I dont know how to use those yet.
        SPIN_NORMAL,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

    
      YQRZ_30->setRange(0 , INT_MAX);
    
    
      YQRZ_30->setIncrement(100);
    

  

  

  

  


    
  FXLabel *YQRZ_33;

  YQRZ_33= 
    new FXLabel(YQRZ_36,
      " MB" ,
      NULL,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        0 |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


      FXHorizontalFrame *YQRZ_45= new
    FXHorizontalFrame( YQRZ_46,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

    
  FXTextField *YQRZ_37;

  destdir = YQRZ_37= 
    new FXTextField(YQRZ_45,
      40 ,
       NULL,
       0,
 
     LAYOUT_LEFT |
     LAYOUT_TOP |
     LAYOUT_SIDE_TOP |
     0 |
     0 |

    FRAME_SUNKEN|FRAME_THICK |
    JUSTIFY_LEFT |
    JUSTIFY_CENTER_Y

        ,

    0,0,0,0, // x,y,w,h

    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD);


    
  FXButton *YQRZ_40;

  YQRZ_40= 
    new FXButton(YQRZ_45,
      "Browse" ,
      NULL,
      this ,
      YQRZ_41 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

      FXGroupBox *YQRZ_74;
  YQRZ_74= new FXGroupBox(YQRZ_149, "End of Files Indicator" , 
        GROUPBOX_TITLE_LEFT |  // FIXME: this shall be configurable
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 |
        FRAME_LINE |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,
     0,0,0,0, // xywh
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

    FXVerticalFrame *YQRZ_73= new
    FXVerticalFrame( YQRZ_74,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        LAYOUT_FILL_Y  ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

     FXHorizontalFrame *YQRZ_63= new
    FXHorizontalFrame( YQRZ_73,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

    
  FXRadioButton * YQRZ_48;
  YQRZ_48= new FXRadioButton(YQRZ_63, "None" , 
               this, YQRZ_49, 
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |
        FRAME_NONE |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

    append( eof_indicator, YQRZ_48, 0 );

    
     YQRZ_48->setCheck(FALSE); 


  

  

  

  


    
  FXRadioButton * YQRZ_53;
  YQRZ_53= new FXRadioButton(YQRZ_63, "Builtin" , 
               this, YQRZ_54, 
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |
        FRAME_NONE |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

    append( eof_indicator, YQRZ_53, 1 );

    
     YQRZ_53->setCheck(FALSE); 


  

  

  

  


    
  FXRadioButton * YQRZ_58;
  YQRZ_58= new FXRadioButton(YQRZ_63, "File Below" , 
               this, YQRZ_59, 
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |
        FRAME_NONE |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

    append( eof_indicator, YQRZ_58, 2 );

    
     YQRZ_58->setCheck(FALSE); 


  

  

  

  


      FXHorizontalFrame *YQRZ_72= new
    FXHorizontalFrame( YQRZ_73,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

    
  FXTextField *YQRZ_64;

  eof_file = YQRZ_64= 
    new FXTextField(YQRZ_72,
      40 ,
       NULL,
       0,
 
     LAYOUT_LEFT |
     LAYOUT_TOP |
     LAYOUT_SIDE_TOP |
     LAYOUT_FILL_X  |
     0 |

    FRAME_SUNKEN|FRAME_THICK |
    JUSTIFY_LEFT |
    JUSTIFY_CENTER_Y

        ,

    0,0,0,0, // x,y,w,h

    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD);


    
  FXButton *YQRZ_67;

  YQRZ_67= 
    new FXButton(YQRZ_72,
      "Browse" ,
      NULL,
      this ,
      YQRZ_68 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

      FXGroupBox *YQRZ_119;
  YQRZ_119= new FXGroupBox(YQRZ_149, "Notification Alarm" , 
        GROUPBOX_TITLE_LEFT |  // FIXME: this shall be configurable
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 |
        FRAME_LINE |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,
     0,0,0,0, // xywh
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

    FXVerticalFrame *YQRZ_118= new
    FXVerticalFrame( YQRZ_119,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        LAYOUT_FILL_Y  ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

     FXHorizontalFrame *YQRZ_90= new
    FXHorizontalFrame( YQRZ_118,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

    
  FXRadioButton * YQRZ_75;
  YQRZ_75= new FXRadioButton(YQRZ_90, "None" , 
               this, YQRZ_76, 
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |
        FRAME_NONE |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

    append( alarm, YQRZ_75, 0 );

    
     YQRZ_75->setCheck(FALSE); 


  

  

  

  


    
  FXRadioButton * YQRZ_80;
  YQRZ_80= new FXRadioButton(YQRZ_90, "Builtin" , 
               this, YQRZ_81, 
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |
        FRAME_NONE |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

    append( alarm, YQRZ_80, 1 );

    
     YQRZ_80->setCheck(FALSE); 


  

  

  

  


    
  FXRadioButton * YQRZ_85;
  YQRZ_85= new FXRadioButton(YQRZ_90, "File Below" , 
               this, YQRZ_86, 
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |
        FRAME_NONE |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

    append( alarm, YQRZ_85, 2 );

    
     YQRZ_85->setCheck(FALSE); 


  

  

  

  


      FXHorizontalFrame *YQRZ_99= new
    FXHorizontalFrame( YQRZ_118,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

    
  FXTextField *YQRZ_91;

  alarm_file = YQRZ_91= 
    new FXTextField(YQRZ_99,
      40 ,
       NULL,
       0,
 
     LAYOUT_LEFT |
     LAYOUT_TOP |
     LAYOUT_SIDE_TOP |
     LAYOUT_FILL_X  |
     0 |

    FRAME_SUNKEN|FRAME_THICK |
    JUSTIFY_LEFT |
    JUSTIFY_CENTER_Y

        ,

    0,0,0,0, // x,y,w,h

    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD);


    
  FXButton *YQRZ_94;

  YQRZ_94= 
    new FXButton(YQRZ_99,
      "Browse" ,
      NULL,
      this ,
      YQRZ_95 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

      FXHorizontalFrame *YQRZ_106= new
    FXHorizontalFrame( YQRZ_118,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

    
  FXLabel *YQRZ_100;

  YQRZ_100= 
    new FXLabel(YQRZ_106,
      "Alarm Player:" ,
      NULL,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        0 |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


    
  FXTextField *YQRZ_103;

  alarm_player = YQRZ_103= 
    new FXTextField(YQRZ_106,
      40 ,
       NULL,
       0,
 
     LAYOUT_LEFT |
     LAYOUT_TOP |
     LAYOUT_SIDE_TOP |
     LAYOUT_FILL_X  |
     0 |

    FRAME_SUNKEN|FRAME_THICK |
    JUSTIFY_LEFT |
    JUSTIFY_CENTER_Y

        ,

    0,0,0,0, // x,y,w,h

    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD);


      FXHorizontalFrame *YQRZ_117= new
    FXHorizontalFrame( YQRZ_118,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

    
  FXButton *YQRZ_107;

  YQRZ_107= 
    new FXButton(YQRZ_117,
      "Test Alarm" ,
      NULL,
      this ,
      YQRZ_108 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

    
  FXButton *YQRZ_112;

  YQRZ_112= 
    new FXButton(YQRZ_117,
      "Stop Alarm" ,
      NULL,
      this ,
      YQRZ_113 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

      FXHorizontalFrame *YQRZ_129= new
    FXHorizontalFrame( YQRZ_149,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

  
  FXLabel *YQRZ_120;

  YQRZ_120= 
    new FXLabel(YQRZ_129,
      "Copy " ,
      NULL,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        0 |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


  
  FXSpinner *YQRZ_123;

  YQRZ_123= gui_check_period =
    new FXSpinner(YQRZ_129, 3,
       // target and selector
       NULL, 0,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |
        0 |
       // there are some options, but I dont know how to use those yet.
        SPIN_NORMAL,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

    
      YQRZ_123->setRange(10 , INT_MAX);
    
    
      YQRZ_123->setIncrement(10);
    

  

  

  

  


  
  FXLabel *YQRZ_126;

  YQRZ_126= 
    new FXLabel(YQRZ_129,
      " files each time." ,
      NULL,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        0 |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


    FXHorizontalFrame *YQRZ_145= new
    FXHorizontalFrame( YQRZ_149,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

  
  FXButton *YQRZ_130;

  YQRZ_130= 
    new FXButton(YQRZ_145,
      "Help" ,
      NULL,
      this ,
      YQRZ_131 ,

        LAYOUT_LEFT  |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

  
  FXButton *YQRZ_135;

  YQRZ_135= 
    new FXButton(YQRZ_145,
      "  Charge   " ,
      NULL,
      this ,
      YQRZ_136 ,

        LAYOUT_CENTER_X  |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  
     FXFontDesc YQRZ_137;
     YQRZ_135->getFont()->getFontDesc(YQRZ_137);
  

  
     YQRZ_137.size= 14 * 10;
  

  

  
    YQRZ_138= new FXFont(FXApp::instance(), YQRZ_137);
    YQRZ_135->setFont(YQRZ_138);
        


         

  
  FXButton *YQRZ_140;

  YQRZ_140= 
    new FXButton(YQRZ_145,
      "Stop" ,
      NULL,
      this ,
      YQRZ_141 ,

        LAYOUT_RIGHT  |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

  
  FXText *YQRZ_146;
  YQRZ_146= status = new FXText(
       YQRZ_149, NULL,0, // target & selector
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 |
        TEXT_FIXEDWRAP |
        VSCROLLER_ALWAYS |
        TEXT_READONLY | TEXT_AUTOSCROLL |
        0,
     0,0,0,0,  // x,y,w,h
             // FIXME: these are given as number constants in the
             // FXText constructor. Nothing symbolic like DEFAULT_PAD
             // etc. is used. So, I will do the same unwillingly.
     3,
     3,
     2,
     2);

  

  

  

  


  
 }
 VFLL1:: ~ VFLL1() { }
 long VFLL1::handle_close
    (FXObject *object,FXSelector sel, void *arg)
 {
     return close_main_window();  
    return 0;
 } 
 

  long VFLL1::YQRZ_13
    (FXObject *object,FXSelector sel, void *arg)
       {  add_dirs();  return 1; }


  

  long VFLL1::YQRZ_18
    (FXObject *object,FXSelector sel, void *arg)
       {   remove_selected();  return 1; }


  

  long VFLL1::YQRZ_23
    (FXObject *object,FXSelector sel, void *arg)
       {  scan_dirs();  return 1; }


    

    


    

  long VFLL1::YQRZ_44
    (FXObject *object,FXSelector sel, void *arg)
       {  select_dest_dir();  return 1; }


    
long VFLL1::YQRZ_52
  (FXObject *object,FXSelector sel, void *arg)
{
  set_value( eof_indicator, 0 );
  /* NOP */
  // FIXME: put the on_click stuff here
  return 1;
}

    
long VFLL1::YQRZ_57
  (FXObject *object,FXSelector sel, void *arg)
{
  set_value( eof_indicator, 1 );
  /* NOP */
  // FIXME: put the on_click stuff here
  return 1;
}

    
long VFLL1::YQRZ_62
  (FXObject *object,FXSelector sel, void *arg)
{
  set_value( eof_indicator, 2 );
  /* NOP */
  // FIXME: put the on_click stuff here
  return 1;
}

    


    

  long VFLL1::YQRZ_71
    (FXObject *object,FXSelector sel, void *arg)
       { 
            FXString fn;
            fn= FXFileDialog::getOpenFilename(this,
                  "Select End-of-Files Indicator",
                 get_home_dir(), "MP3 Files (*.mp3,*.MP3)", 0);
            if (!compare(fn, FXString::null)) return 1;
            eof_file->setText(fn);
           return 1; }


    
long VFLL1::YQRZ_79
  (FXObject *object,FXSelector sel, void *arg)
{
  set_value( alarm, 0 );
  /* NOP */
  // FIXME: put the on_click stuff here
  return 1;
}

    
long VFLL1::YQRZ_84
  (FXObject *object,FXSelector sel, void *arg)
{
  set_value( alarm, 1 );
  /* NOP */
  // FIXME: put the on_click stuff here
  return 1;
}

    
long VFLL1::YQRZ_89
  (FXObject *object,FXSelector sel, void *arg)
{
  set_value( alarm, 2 );
  /* NOP */
  // FIXME: put the on_click stuff here
  return 1;
}

    


    

  long VFLL1::YQRZ_98
    (FXObject *object,FXSelector sel, void *arg)
       { 
            FXString fn;
            fn= FXFileDialog::getOpenFilename(this, "Select Alarm Sound File",
                 get_home_dir(), "Sound Files (*.mp3,*.wav)", 0);
            if (!compare(fn, FXString::null)) return 1;
            alarm_file->setText(fn);
           return 1; }


    


    

  long VFLL1::YQRZ_111
    (FXObject *object,FXSelector sel, void *arg)
       {  play_alarm();  return 1; }


    

  long VFLL1::YQRZ_116
    (FXObject *object,FXSelector sel, void *arg)
       {  stop_alarm();  return 1; }


    

  long VFLL1::YQRZ_134
    (FXObject *object,FXSelector sel, void *arg)
       {  show_help_dialog();  return 1; }


  

  long VFLL1::YQRZ_139
    (FXObject *object,FXSelector sel, void *arg)
       {  charge();  return 1; }


  

  long VFLL1::YQRZ_144
    (FXObject *object,FXSelector sel, void *arg)
       {  stop_charging();  return 1; }


   


 
 class VFLL2: public FXTopWindow
 {
  FXDECLARE(VFLL2)
 public:
  enum { ID_FIRST= FXTopWindow::ID_LAST,
         ID_SELF,
           YQRZ_152,

      YQRZ_157,


  YQRZ_160,

   YQRZ_165,

   YQRZ_170,

    };
     FXDirList *dirs;
   
       long YQRZ_155(FXObject *object, FXSelector sel, void *arg);

 long YQRZ_158
       (FXObject *object,FXSelector sel, void *arg);
FXTextField *dir_name;

 
  long YQRZ_163(FXObject *object,FXSelector sel, void *arg);
  

  
  long YQRZ_168(FXObject *object,FXSelector sel, void *arg);
  

  
  long YQRZ_173(FXObject *object,FXSelector sel, void *arg);
  

   
  VFLL2(FXWindow* own,const FXString& name,
         FXIcon *ic,FXIcon *mi,FXuint opts,
         FXint x,FXint y,FXint w,FXint h,
         FXint pl,FXint pr,FXint pt,FXint pb,
         FXint hs,FXint vs);
  virtual ~VFLL2();
  VFLL2() : FXTopWindow() { }
  void dialog_done(int value);
  long handle_close(FXObject *object,FXSelector sel, void *arg);
 };

 FXDEFMAP(VFLL2) YQRZ_177 []= {
  FXMAPFUNC(SEL_CLOSE, VFLL2::ID_SELF, VFLL2::handle_close),
  
  
    FXMAPFUNC(SEL_COMMAND, VFLL2::YQRZ_152, VFLL2::YQRZ_155), 

  
   FXMAPFUNC(SEL_COMMAND, VFLL2::YQRZ_157, VFLL2::YQRZ_158),

  
    FXMAPFUNC(SEL_COMMAND, VFLL2::YQRZ_160, VFLL2::YQRZ_163),

   
    FXMAPFUNC(SEL_COMMAND, VFLL2::YQRZ_165, VFLL2::YQRZ_168),

   
    FXMAPFUNC(SEL_COMMAND, VFLL2::YQRZ_170, VFLL2::YQRZ_173),

   
 }; 

 FXIMPLEMENT(VFLL2, FXTopWindow, YQRZ_177, ARRAYNUMBER(YQRZ_177))
 
 // FIXME: the argument names must be changed so that
 // we can have widgets called 'y' etc.
 VFLL2::VFLL2(
         FXWindow* YQRZ_178,const FXString& YQRZ_179,
         FXIcon *YQRZ_180,FXIcon *YQRZ_181,FXuint YQRZ_182,
         FXint YQRZ_183,FXint YQRZ_184,FXint YQRZ_185,FXint YQRZ_186,
         FXint YQRZ_187,FXint YQRZ_188,FXint YQRZ_189,FXint YQRZ_190,
         FXint YQRZ_191,FXint YQRZ_192) :
               FXTopWindow(YQRZ_178,YQRZ_179, YQRZ_180,YQRZ_181,YQRZ_182,
                            YQRZ_183,YQRZ_184,YQRZ_185,YQRZ_186,
                            YQRZ_187,YQRZ_188,YQRZ_189,YQRZ_190,
                            YQRZ_191,YQRZ_192)
 {
   setTarget(this);
   setSelector(ID_SELF);
     FXVerticalFrame *YQRZ_175= new
    FXVerticalFrame( this,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        LAYOUT_FILL_Y  ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

 
  FXDirList *YQRZ_151;
  YQRZ_151= dirs = new FXDirList(YQRZ_175,
           this ,
           YQRZ_152 ,
             TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|TREELIST_BROWSESELECT|
             DIRLIST_NO_OWN_ASSOC |
             0 |
             0 |

             LAYOUT_LEFT |
             LAYOUT_TOP |
             LAYOUT_SIDE_TOP |
             LAYOUT_FILL_X  |
             LAYOUT_FILL_Y  |
             FRAME_RAISED|FRAME_THICK 
           ,0,0,0,0);  // xywh

  

  

  

  


 
  FXTextField *YQRZ_156;

  dir_name = YQRZ_156= 
    new FXTextField(YQRZ_175,
      40,
      this ,
      YQRZ_157 ,
 
     LAYOUT_LEFT |
     LAYOUT_TOP |
     LAYOUT_SIDE_TOP |
     LAYOUT_FILL_X  |
     0 |

    FRAME_SUNKEN|FRAME_THICK |
    JUSTIFY_LEFT |
    JUSTIFY_CENTER_Y

        ,

    0,0,0,0, // x,y,w,h

    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD);


   FXHorizontalFrame *YQRZ_174= new
    FXHorizontalFrame( YQRZ_175,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

  
  FXButton *YQRZ_159;

  YQRZ_159= 
    new FXButton(YQRZ_174,
      "Include" ,
      NULL,
      this ,
      YQRZ_160 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

  
  FXButton *YQRZ_164;

  YQRZ_164= 
    new FXButton(YQRZ_174,
      "Exclude" ,
      NULL,
      this ,
      YQRZ_165 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

  
  FXButton *YQRZ_169;

  YQRZ_169= 
    new FXButton(YQRZ_174,
      "Done" ,
      NULL,
      this ,
      YQRZ_170 ,

        LAYOUT_RIGHT  |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

   
 }
 VFLL2:: ~ VFLL2() { }
 long VFLL2::handle_close
    (FXObject *object,FXSelector sel, void *arg)
 {
     dialog_done(0); return 1;  
    return 0;
 } 
 void VFLL2::dialog_done(int value)
 {
   FXApp::instance()->stopModal(value);
 }
 
  
long VFLL2::YQRZ_155
     (FXObject *object, FXSelector sel, void *arg)
{
   
       FXTreeItem *item= (FXTreeItem*) arg;
       if (!item) return 1;
       dir_name->setText(dirs->getItemPathname(item));
    
   return 1;
}
  

 

  long VFLL2::YQRZ_158
    (FXObject *object,FXSelector sel, void *arg)
       { 
       dirs->setDirectory(dir_name->getText());
     return 1; }


 

  long VFLL2::YQRZ_163
    (FXObject *object,FXSelector sel, void *arg)
       {  include_dir(dir_name->getText().text()); return 1; }


  

  long VFLL2::YQRZ_168
    (FXObject *object,FXSelector sel, void *arg)
       {  exclude_dir(dir_name->getText().text()); return 1; }


  

  long VFLL2::YQRZ_173
    (FXObject *object,FXSelector sel, void *arg)
       {  dialog_done(0);  return 1; }


   


 
 class VFLL3: public FXTopWindow
 {
  FXDECLARE(VFLL3)
 public:
  enum { ID_FIRST= FXTopWindow::ID_LAST,
         ID_SELF,
           YQRZ_194,

      YQRZ_199,


  YQRZ_202,

   YQRZ_207,

    };
     FXDirList *dirs;
   
       long YQRZ_197(FXObject *object, FXSelector sel, void *arg);

 long YQRZ_200
       (FXObject *object,FXSelector sel, void *arg);
FXTextField *dir_name;

 
  long YQRZ_205(FXObject *object,FXSelector sel, void *arg);
  

  
  long YQRZ_210(FXObject *object,FXSelector sel, void *arg);
  

   
  VFLL3(FXWindow* own,const FXString& name,
         FXIcon *ic,FXIcon *mi,FXuint opts,
         FXint x,FXint y,FXint w,FXint h,
         FXint pl,FXint pr,FXint pt,FXint pb,
         FXint hs,FXint vs);
  virtual ~VFLL3();
  VFLL3() : FXTopWindow() { }
  void dialog_done(int value);
  long handle_close(FXObject *object,FXSelector sel, void *arg);
 };

 FXDEFMAP(VFLL3) YQRZ_214 []= {
  FXMAPFUNC(SEL_CLOSE, VFLL3::ID_SELF, VFLL3::handle_close),
  
  
    FXMAPFUNC(SEL_COMMAND, VFLL3::YQRZ_194, VFLL3::YQRZ_197), 

  
   FXMAPFUNC(SEL_COMMAND, VFLL3::YQRZ_199, VFLL3::YQRZ_200),

  
    FXMAPFUNC(SEL_COMMAND, VFLL3::YQRZ_202, VFLL3::YQRZ_205),

   
    FXMAPFUNC(SEL_COMMAND, VFLL3::YQRZ_207, VFLL3::YQRZ_210),

   
 }; 

 FXIMPLEMENT(VFLL3, FXTopWindow, YQRZ_214, ARRAYNUMBER(YQRZ_214))
 
 // FIXME: the argument names must be changed so that
 // we can have widgets called 'y' etc.
 VFLL3::VFLL3(
         FXWindow* YQRZ_215,const FXString& YQRZ_216,
         FXIcon *YQRZ_217,FXIcon *YQRZ_218,FXuint YQRZ_219,
         FXint YQRZ_220,FXint YQRZ_221,FXint YQRZ_222,FXint YQRZ_223,
         FXint YQRZ_224,FXint YQRZ_225,FXint YQRZ_226,FXint YQRZ_227,
         FXint YQRZ_228,FXint YQRZ_229) :
               FXTopWindow(YQRZ_215,YQRZ_216, YQRZ_217,YQRZ_218,YQRZ_219,
                            YQRZ_220,YQRZ_221,YQRZ_222,YQRZ_223,
                            YQRZ_224,YQRZ_225,YQRZ_226,YQRZ_227,
                            YQRZ_228,YQRZ_229)
 {
   setTarget(this);
   setSelector(ID_SELF);
     FXVerticalFrame *YQRZ_212= new
    FXVerticalFrame( this,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        LAYOUT_FILL_Y  ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

 
  FXDirList *YQRZ_193;
  YQRZ_193= dirs = new FXDirList(YQRZ_212,
           this ,
           YQRZ_194 ,
             TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|TREELIST_BROWSESELECT|
             DIRLIST_NO_OWN_ASSOC |
             0 |
             0 |

             LAYOUT_LEFT |
             LAYOUT_TOP |
             LAYOUT_SIDE_TOP |
             LAYOUT_FILL_X  |
             LAYOUT_FILL_Y  |
             FRAME_RAISED|FRAME_THICK 
           ,0,0,0,0);  // xywh

  

  

  

  


 
  FXTextField *YQRZ_198;

  dir_name = YQRZ_198= 
    new FXTextField(YQRZ_212,
      40,
      this ,
      YQRZ_199 ,
 
     LAYOUT_LEFT |
     LAYOUT_TOP |
     LAYOUT_SIDE_TOP |
     LAYOUT_FILL_X  |
     0 |

    FRAME_SUNKEN|FRAME_THICK |
    JUSTIFY_LEFT |
    JUSTIFY_CENTER_Y

        ,

    0,0,0,0, // x,y,w,h

    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD,
    DEFAULT_PAD);


   FXHorizontalFrame *YQRZ_211= new
    FXHorizontalFrame( YQRZ_212,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

  
  FXButton *YQRZ_201;

  YQRZ_201= 
    new FXButton(YQRZ_211,
      "Select" ,
      NULL,
      this ,
      YQRZ_202 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

  
  FXButton *YQRZ_206;

  YQRZ_206= 
    new FXButton(YQRZ_211,
      "Cancel" ,
      NULL,
      this ,
      YQRZ_207 ,

        LAYOUT_RIGHT  |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

   
 }
 VFLL3:: ~ VFLL3() { }
 long VFLL3::handle_close
    (FXObject *object,FXSelector sel, void *arg)
 {
     dialog_done(0); return 1;  
    return 0;
 } 
 void VFLL3::dialog_done(int value)
 {
   FXApp::instance()->stopModal(value);
 }
 
  
long VFLL3::YQRZ_197
     (FXObject *object, FXSelector sel, void *arg)
{
   
       FXTreeItem *item= (FXTreeItem*) arg;
       if (!item) return 1;
       dir_name->setText(dirs->getItemPathname(item));
    
   return 1;
}
  

 

  long VFLL3::YQRZ_200
    (FXObject *object,FXSelector sel, void *arg)
       { 
       dirs->setDirectory(dir_name->getText());
     return 1; }


 

  long VFLL3::YQRZ_205
    (FXObject *object,FXSelector sel, void *arg)
       {  set_dest_dir(dir_name->getText().text());
                   dialog_done(0);  return 1; }


  

  long VFLL3::YQRZ_210
    (FXObject *object,FXSelector sel, void *arg)
       {  dialog_done(0);  return 1; }


   


 
 class VFLL4: public FXTopWindow
 {
  FXDECLARE(VFLL4)
 public:
  enum { ID_FIRST= FXTopWindow::ID_LAST,
         ID_SELF,
          YQRZ_234,

   YQRZ_239,

   YQRZ_244,

   YQRZ_249,

    };
  
  FXText *text;

 
  long YQRZ_237(FXObject *object,FXSelector sel, void *arg);
  

  
  long YQRZ_242(FXObject *object,FXSelector sel, void *arg);
  

  
  long YQRZ_247(FXObject *object,FXSelector sel, void *arg);
  

  
  long YQRZ_252(FXObject *object,FXSelector sel, void *arg);
  

   
  VFLL4(FXWindow* own,const FXString& name,
         FXIcon *ic,FXIcon *mi,FXuint opts,
         FXint x,FXint y,FXint w,FXint h,
         FXint pl,FXint pr,FXint pt,FXint pb,
         FXint hs,FXint vs);
  virtual ~VFLL4();
  VFLL4() : FXTopWindow() { }
  void dialog_done(int value);
  long handle_close(FXObject *object,FXSelector sel, void *arg);
 };

 FXDEFMAP(VFLL4) YQRZ_256 []= {
  FXMAPFUNC(SEL_CLOSE, VFLL4::ID_SELF, VFLL4::handle_close),
   
    FXMAPFUNC(SEL_COMMAND, VFLL4::YQRZ_234, VFLL4::YQRZ_237),

   
    FXMAPFUNC(SEL_COMMAND, VFLL4::YQRZ_239, VFLL4::YQRZ_242),

   
    FXMAPFUNC(SEL_COMMAND, VFLL4::YQRZ_244, VFLL4::YQRZ_247),

   
    FXMAPFUNC(SEL_COMMAND, VFLL4::YQRZ_249, VFLL4::YQRZ_252),

   
 }; 

 FXIMPLEMENT(VFLL4, FXTopWindow, YQRZ_256, ARRAYNUMBER(YQRZ_256))
 
 // FIXME: the argument names must be changed so that
 // we can have widgets called 'y' etc.
 VFLL4::VFLL4(
         FXWindow* YQRZ_257,const FXString& YQRZ_258,
         FXIcon *YQRZ_259,FXIcon *YQRZ_260,FXuint YQRZ_261,
         FXint YQRZ_262,FXint YQRZ_263,FXint YQRZ_264,FXint YQRZ_265,
         FXint YQRZ_266,FXint YQRZ_267,FXint YQRZ_268,FXint YQRZ_269,
         FXint YQRZ_270,FXint YQRZ_271) :
               FXTopWindow(YQRZ_257,YQRZ_258, YQRZ_259,YQRZ_260,YQRZ_261,
                            YQRZ_262,YQRZ_263,YQRZ_264,YQRZ_265,
                            YQRZ_266,YQRZ_267,YQRZ_268,YQRZ_269,
                            YQRZ_270,YQRZ_271)
 {
   setTarget(this);
   setSelector(ID_SELF);
     FXVerticalFrame *YQRZ_254= new
    FXVerticalFrame( this,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        LAYOUT_FILL_Y  ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

 
  FXText *YQRZ_230;
  YQRZ_230= text = new FXText(
       YQRZ_254, NULL,0, // target & selector
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        LAYOUT_FILL_Y  |
        TEXT_WORDWRAP |
        VSCROLLER_ALWAYS |
        TEXT_READONLY | 
        0,
     0,0,0,0,  // x,y,w,h
             // FIXME: these are given as number constants in the
             // FXText constructor. Nothing symbolic like DEFAULT_PAD
             // etc. is used. So, I will do the same unwillingly.
     3,
     3,
     2,
     2);

  
     FXFontDesc YQRZ_231;
     YQRZ_230->getFont()->getFontDesc(YQRZ_231);
  

  

  
     strcpy(YQRZ_231.face, "mono");
  

  
    YQRZ_232= new FXFont(FXApp::instance(), YQRZ_231);
    YQRZ_230->setFont(YQRZ_232);
       


   FXHorizontalFrame *YQRZ_253= new
    FXHorizontalFrame( YQRZ_254,
        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        LAYOUT_FILL_X  |
        0 ,
        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_SPACING,
        DEFAULT_SPACING);

  
  FXButton *YQRZ_233;

  YQRZ_233= 
    new FXButton(YQRZ_253,
      "About" ,
      NULL,
      this ,
      YQRZ_234 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

  
  FXButton *YQRZ_238;

  YQRZ_238= 
    new FXButton(YQRZ_253,
      "Licensing" ,
      NULL,
      this ,
      YQRZ_239 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

  
  FXButton *YQRZ_243;

  YQRZ_243= 
    new FXButton(YQRZ_253,
      "View Readme" ,
      NULL,
      this ,
      YQRZ_244 ,

        LAYOUT_LEFT |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

  
  FXButton *YQRZ_248;

  YQRZ_248= 
    new FXButton(YQRZ_253,
      "Done" ,
      NULL,
      this ,
      YQRZ_249 ,

        LAYOUT_RIGHT  |
        LAYOUT_TOP |
        LAYOUT_SIDE_TOP |
        0 |
        0 |

        FRAME_RAISED|FRAME_THICK |
        ICON_BEFORE_TEXT |
        JUSTIFY_CENTER_X |
        JUSTIFY_CENTER_Y ,

        0,0,0,0, // x,y,w,h
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD,
        DEFAULT_PAD);

  

  

  

  


         

   
 }
 VFLL4:: ~ VFLL4() { }
 long VFLL4::handle_close
    (FXObject *object,FXSelector sel, void *arg)
 {
     dialog_done(0); return 1;  
    return 0;
 } 
 void VFLL4::dialog_done(int value)
 {
   FXApp::instance()->stopModal(value);
 }
 

  long VFLL4::YQRZ_237
    (FXObject *object,FXSelector sel, void *arg)
       {  text->setText(help_about);  return 1; }


  

  long VFLL4::YQRZ_242
    (FXObject *object,FXSelector sel, void *arg)
       {  text->setText(help_license);  return 1; }


  

  long VFLL4::YQRZ_247
    (FXObject *object,FXSelector sel, void *arg)
       {  text->setText(help_readme);  return 1; }


  

  long VFLL4::YQRZ_252
    (FXObject *object,FXSelector sel, void *arg)
       {  dialog_done(0);  return 1; }


   


  


 gui_main_t *gui_c_main()
 {
    // ffo -> fox form object
    // ifo -> interface form object 
   VFLL1 *ffo;
   gui_main_t * ifo;
   ffo= new VFLL1(
       FXApp::instance(),
       "MP3 Re-charger" ,
       NULL, NULL, // FIXME: icons
       DECOR_ALL ,
       0, 0,
       420 , 570 ,
       0, 0,
       0, 0,
       0, 0);
   // If the application has not been created, this will done
   // when the application gets created. Otherwise, we have
   // to explicitly do this.
   if (app_created) 
     ffo->create();
   
   ffo->show(PLACEMENT_SCREEN);

   ifo= (typeof(ifo)) malloc(sizeof(*ifo));
   ifo->self= ffo;

 ifo->eof_indicator.widget= ffo->eof_indicator;
 ifo->eof_indicator.cls= &gui_radio_group_class;





 ifo->alarm.widget= ffo->alarm;
 ifo->alarm.cls= &gui_radio_group_class;






   
    ifo->source_dirs.cls= & gui_slist_class;
    ifo->source_dirs.widget= ffo->source_dirs;
   


 




  



  



   
     ifo->size_limit_on.widget= ffo->size_limit_on;
     ifo->size_limit_on.cls= &gui_check_box_class;
  


    ifo->size_limit.widget= ffo->size_limit;
 ifo->size_limit.cls= &gui_spinner_class;



    

   
 ifo->destdir.widget= ffo->destdir;
 ifo->destdir.cls= &gui_text_entry_class;



    



 
 ifo->eof_file.widget= ffo->eof_file;
 ifo->eof_file.cls= &gui_text_entry_class;


    



 
 ifo->alarm_file.widget= ffo->alarm_file;
 ifo->alarm_file.cls= &gui_text_entry_class;


    



   

    
 ifo->alarm_player.widget= ffo->alarm_player;
 ifo->alarm_player.cls= &gui_text_entry_class;


   



    



 

  ifo->gui_check_period.widget= ffo->gui_check_period;
 ifo->gui_check_period.cls= &gui_spinner_class;


  

 



  



  



 

  ifo->status.widget= ffo->status;
  ifo->status.cls= &gui_message_console_class;



  
   return ifo;
 }


  

 gui_dir_selector_t *gui_c_dir_selector(void *owner)
 {
    // ffo -> fox form object
    // ifo -> interface form object 
   VFLL2 *ffo;
   gui_dir_selector_t * ifo;
   ffo= new VFLL2(
       (FXWindow*) owner,
       "Select Directories" ,
       NULL, NULL, // FIXME: icons
       DECOR_ALL ,
       0, 0,
       400, 400,
       0, 0,
       0, 0,
       0, 0);

   ifo= (typeof(ifo)) malloc(sizeof(*ifo));
   ifo->self= ffo;
   ifo->cls= (YQRZ_176*) &gui_dialog_class;
  
    ifo->dirs.widget= ffo->dirs;
    ifo->dirs.cls= &gui_dir_list_class;
  

 
 ifo->dir_name.widget= ffo->dir_name;
 ifo->dir_name.cls= &gui_text_entry_class;


 



  



  



 
   return ifo;
 }


 

 gui_dest_selector_t *gui_c_dest_selector(void *owner)
 {
    // ffo -> fox form object
    // ifo -> interface form object 
   VFLL3 *ffo;
   gui_dest_selector_t * ifo;
   ffo= new VFLL3(
       (FXWindow*) owner,
       "Select Destination Directory" ,
       NULL, NULL, // FIXME: icons
       DECOR_ALL ,
       0, 0,
       400, 400,
       0, 0,
       0, 0,
       0, 0);

   ifo= (typeof(ifo)) malloc(sizeof(*ifo));
   ifo->self= ffo;
   ifo->cls= (YQRZ_213*) &gui_dialog_class;
  
    ifo->dirs.widget= ffo->dirs;
    ifo->dirs.cls= &gui_dir_list_class;
  
 
 ifo->dir_name.widget= ffo->dir_name;
 ifo->dir_name.cls= &gui_text_entry_class;


 



  



 
   return ifo;
 }



 gui_help_t *gui_c_help(void *owner)
 {
    // ffo -> fox form object
    // ifo -> interface form object 
   VFLL4 *ffo;
   gui_help_t * ifo;
   ffo= new VFLL4(
       (FXWindow*) owner,
       "CMP3 Help" ,
       NULL, NULL, // FIXME: icons
       DECOR_ALL ,
       0, 0,
       630 , 400 ,
       0, 0,
       0, 0,
       0, 0);

   ifo= (typeof(ifo)) malloc(sizeof(*ifo));
   ifo->self= ffo;
   ifo->cls= (YQRZ_255*) &gui_dialog_class;


  ifo->text.widget= ffo->text;
  ifo->text.cls= &gui_text_viewer_class;



 



  



  



  



 
   return ifo;
 }

   

static FXApp *app;

void gui_init(int *argc,char **argv)
{
  app= new FXApp("YAK", "KUT");
  app->init(*argc, argv);
}

void gui_run()
{
  app->create();
   // YQRZ_8->create(); 
        // YQRZ_138->create(); 
         // YQRZ_232->create(); 
        
  app_created= 1;
  app->run();
}

void gui_flush()
{
  app->runWhileEvents();
}

