#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <gcrypt.h>
#include <errno.h>
#include <signal.h>
#include <sys/fcntl.h>
#include <unistd.h>
#include "config.h"
#include "gui.h"
#include "actions.h"
#include "proc.h"
#include "ding.h"
#include "help.h"

gui_main_t *gui;
gui_dir_selector_t *dir_selector;
gui_dest_selector_t *dest_selector;
cfg_config_t config;

typedef struct {
  char *name, *hash;
  int count, aux;
} mp3file_t;


#ifndef _buffer_type_mp3buffer_t_defined
#define _buffer_type_mp3buffer_t_defined

typedef struct {
    struct mp3buffer_t_node *first, *last;
} mp3buffer_t;
#endif

 mp3buffer_t *mp3buffer_new() ;
  void mp3buffer_append(mp3buffer_t *B, mp3file_t d) ;
  mp3file_t *mp3buffer_collect(mp3buffer_t *B, int *len) ;
  mp3file_t *mp3buffer_collectd(mp3buffer_t *B, int *len) ;
  mp3file_t *mp3buffer_free(mp3buffer_t *B) ;
  
#ifndef _buffer_type_mp3buffer_t_defined
#define _buffer_type_mp3buffer_t_defined

typedef struct {
    struct mp3buffer_t_node *first, *last;
} mp3buffer_t;
#endif


struct mp3buffer_t_node {
   struct mp3buffer_t_node *next;
   int len;
   mp3file_t data[100];
};

 static struct mp3buffer_t_node *mp3buffer_t_newblock() ;
  mp3buffer_t *mp3buffer_new() ;
  void mp3buffer_append(mp3buffer_t *B, mp3file_t d) ;
  mp3file_t *mp3buffer_collect(mp3buffer_t *B, int *len) ;
  mp3file_t *mp3buffer_collectd(mp3buffer_t *B, int *len) ;
  mp3file_t *mp3buffer_free(mp3buffer_t *B) ;
  
static struct mp3buffer_t_node *mp3buffer_t_newblock()
{
  struct mp3buffer_t_node *R= malloc(sizeof(*R));
  R->len= 0;
  R->next= 0;
  return R;
}

mp3buffer_t *mp3buffer_new()
{
   mp3buffer_t *B= malloc(sizeof(*B));
   B->first= B->last= mp3buffer_t_newblock();
   return B;
}

void mp3buffer_append(mp3buffer_t *B, mp3file_t d)
{
   struct mp3buffer_t_node *L= B->last;
   if (L->len==100)
     B->last= L= B->last->next= mp3buffer_t_newblock();
   L->data[L->len++]= d;
}

mp3file_t *mp3buffer_collect(mp3buffer_t *B, int *len)
{
   int S= 0;
   struct mp3buffer_t_node *N;
   mp3file_t *R;
   for(N=B->first;N;N=N->next)
     S+= N->len;
   R= malloc(S*sizeof(mp3file_t));
   S= 0;
   for(N=B->first;N;N=N->next)
     { memcpy(R+S, N->data, sizeof(mp3file_t)*N->len);
       S+= N->len; }
   if (len) *len= S;
   return R;
}

mp3file_t *mp3buffer_collectd(mp3buffer_t *B, int *len)
{
   mp3file_t *R;
   R= mp3buffer_collect(B, len);
   mp3buffer_free(B);
   return R;
}

mp3file_t *mp3buffer_free(mp3buffer_t *B)
{
   struct mp3buffer_t_node *N,*T;
   for(N=B->first;N;N=T)
    { T= N->next; free(N); }
   free(B);
}


typedef struct {
  int selts;
  int tos;
  char* *elts; 
} dirstack_t;

dirstack_t *dirstack_new(int initial);
int dirstack_empty(dirstack_t *S);
void dirstack_push(dirstack_t *S, char* V);
int dirstack_pop(dirstack_t *S, char** R);


dirstack_t *dirstack_new(int initial)
{
  dirstack_t *S;
  S= malloc(sizeof(*S));
  S->selts= initial;
  S->tos= -1;
  S->elts= malloc(sizeof(char*)*S->selts); 
  return S;
}

int dirstack_empty(dirstack_t *S)
  { return S->tos==-1; }

void dirstack_push(dirstack_t *S, char* V)
{
  if(S->tos==S->selts-1) {
    S->selts+= S->selts/2;
    S->elts= realloc(S->elts, sizeof(char*)*S->selts);
  }
  S->elts[++S->tos]= V;
}

int dirstack_pop(dirstack_t *S, char** R)
{
  if (S->tos==-1) return 0;
  if (R) *R= S->elts[S->tos];
  S->tos--;
  return 1;
}


void include_dir(const char *dn);
void exclude_dir(const char *dn);
void remove_selected();
void add_dirs();
void print_lists();
void scan_dirs();
void select_dest_dir();
void set_dest_dir(const char *dest);
char *get_home_dir();
void play_alarm();
void stop_alarm();
void charge() ;
void stop_charging();
int close_main_window();
void show_help_dialog();

typedef struct {
   int magic, nfiles, size_str, size_full;
} db_header_t;

#define CMP3_MAGIC  ('C' | 'M'<<8 | 'P'<<16 | '3'<<24)

static int alarm_player_id = -1;

static int copy_errno;

static int charging;


static unsigned long long copy_limit, copied_bytes;


static int stop_charge;


static void show_settings() ;
static char *config_file_name();
static void remove_from_array
   (char *v, int *size, int *len, char ***A);
static void add_to_array
   (char *v, int *size, int *len, char ***A);
static void add_dir(char *dir,char *kind);
static void unify_array(char **src, int slen, char ***dst, int *dlen);
static int get_sources(char ***src,int *nsrc,char ***ex, int *nex);
static void prepare_buffers
   (mp3buffer_t **Rbuffer, dirstack_t **Rdirs,
    char **sources, int nsources);
static void scan_all 
    (mp3file_t *db, int ndb,
     mp3buffer_t *buffer, dirstack_t *dirs,
     char **excl, int nexcl);
static void scan_directory
    (mp3buffer_t* buffer, dirstack_t *dirs, char *name, 
     mp3file_t *db, int ndb);
static char *combine(char *dir,char *file);
static int has_ext(char *name, char *ext);
static void make_path
   (char **dst, int *dsize,
    char *dir, char *file);
static int find_file(mp3file_t *db,int ndb, char *name);
static void flatten_buffer
  (mp3buffer_t *buffer, 
   mp3file_t **Rarray, int *Rn,
   char **Rstrarea);
static void compute_md5_hash
   (char *path,char *sum);
static void merge_databases
  (mp3file_t *db, int ndb, mp3buffer_t *buffer,
   mp3file_t **Rdb, int *Rndb,char **Rstrarea);
static int sort_by_aux(mp3file_t *db,int ndb);
static void sort_by_hash(mp3file_t *db, int n);
static mp3file_t* find_file_by_hash
    (mp3file_t *db,int n, char *hash);
static void write_database(mp3file_t *files, int nf);
static int write_bytes(FILE *f, void *buffer, int len);
static void read_database(mp3file_t **Rfiles, int *Rnfiles);
static int read_bytes(FILE *f, void *buffer, int len);
static char *db_file_name();
char **get_player_args();
static void play_builtin_alarm();
char *get_destination_dir();
static int isspc(unsigned int k);
static void remove_files(char *dest_dir);
static int copy_file(char *src, char *destdir, char *destname);
static float *compute_likeliness(mp3file_t *files, int nfiles);
static float *compute_likeliness_totals(float *likely, int n);
static int find_file_by_likeliness
    (float *likeliness, float *tree, int n, double G);
static char *choose_file_to_copy
     (mp3file_t *files, float *likeliness, float *tree, int nfiles);
static void get_copy_limit() ;
static int copy_eof_indicator(char *dest) ;
static void write_config();
static void write_int(FILE *f, char *name, int V);
static void write_str(FILE *f, char *name, char* value);
static void check_and_write(FILE *f, char *name, char *value);
static void trim_string(char *value);

void include_dir(const char *dn)
{
  char *S= strdup(dn);
  gui_remove(gui->source_dirs, S);
  remove_from_array(S,&config.S_source_dir,
                    &config.N_source_dir,
                    &config.source_dir);
  remove_from_array(S,&config.S_exclude_dir,
                    &config.N_exclude_dir,
                    &config.exclude_dir);
  add_to_array(S,&config.S_source_dir,
                    &config.N_source_dir,
                    &config.source_dir);
  add_dir(S,"    ");
}

void exclude_dir(const char *dn)
{
  char *S= strdup(dn);
  gui_remove(gui->source_dirs, S);
  remove_from_array(S,&config.S_source_dir,
                    &config.N_source_dir,
                    &config.source_dir);
  remove_from_array(S,&config.S_exclude_dir,
                    &config.N_exclude_dir,
                    &config.exclude_dir);
  add_to_array(S,&config.S_exclude_dir,
                    &config.N_exclude_dir,
                    &config.exclude_dir);
  add_dir(S,"EXC ");
}

void remove_selected()
{
  char *S;
  if (!gui_list_get_selected(gui->source_dirs, &S)) return ;
  gui_remove(gui->source_dirs, S);
  remove_from_array(S,&config.S_source_dir,
                    &config.N_source_dir,
                    &config.source_dir);
  remove_from_array(S,&config.S_exclude_dir,
                    &config.N_exclude_dir,
                    &config.exclude_dir);
}


void add_dirs()
{
   char *home;
   dir_selector= gui_c_dir_selector(gui->self);
   home= getenv("HOME");
   if (!home) home= "/home";
   gui_set_text(dir_selector->dir_name, home);
   gui_set_current_dir(dir_selector->dirs, home);
   gui_execute(*dir_selector);
   gui_close(*dir_selector);
}

void print_lists()
{
   int i;
   printf("SOURCE: (%d) (%d)\n", config.S_source_dir, config.N_source_dir);
   for(i=0;i<config.N_source_dir;i++)
     printf("%s\n",config.source_dir[i]);
   printf("EXCLUDE: (%d) (%d)\n", config.S_exclude_dir, config.N_exclude_dir);
   for(i=0;i<config.N_exclude_dir;i++)
     printf("%s\n",config.exclude_dir[i]);
}


void scan_dirs()
{
  char **sources, **excl;
  int nsources, nexcl;
  mp3buffer_t *buffer;
  dirstack_t *dirs;
  mp3file_t *db=0, *new_db; int ndb,nndb;
  char *strarea;

  if (get_sources(&sources, &nsources, &excl, &nexcl)) return ;
  prepare_buffers(&buffer, &dirs, sources, nsources);

  read_database(&db, &ndb);

  scan_all(db, ndb, buffer, dirs, excl, nexcl);
  merge_databases(db, ndb, buffer, &new_db, &nndb, &strarea);

  write_database(new_db, nndb);

  if (db) free(db);
  free(new_db); free(strarea);
  // play alarm here.
}

void select_dest_dir()
{
   dest_selector= gui_c_dest_selector(gui->self);
   gui_set_text(dest_selector->dir_name, "/");
   gui_set_current_dir(dest_selector->dirs, "/");
   gui_execute(*dest_selector);
   gui_close(*dest_selector);
}

void set_dest_dir(const char *dest)
{
  gui_set_text(gui->destdir,(char*) dest);
}

char *get_home_dir()
{
  char *home= getenv("HOME");
  if (home) return home;
  return "/home";
}


void play_alarm()
{
   int i;
   int af;
   char **player;
   char *alarm_file;
   if (alarm_player_id>0) {
     gui_print(gui->status,"Already playing alarm.\n");
     return ;
   }
   gui_get_value(gui->alarm,&af);
   switch(af) {
   case 0: gui_print(gui->status, "Check settings. Alarm is disabled.\n");
           return ;
   case 1: play_builtin_alarm(); return;
   }
   player= get_player_args();
   if (!player) return ;
   alarm_file= gui_get_text(gui->alarm_file);
   if (!alarm_file[0])
     { gui_print(gui->status, "No alarm file given.\n");
       free(alarm_file);
       return ; }
   for(i=0;player[i];i++)
     if (!strcmp(player[i], "@")) {
       free(player[i]);
       player[i]= strdup(alarm_file);
     }
   alarm_player_id= proc_start(player[0], player,0,0);
   if (alarm_player_id<=0) {
     gui_print(gui->status, "Could not execute alarm player:\n  %s\n",
         strerror(-alarm_player_id));
   }
   for(i=0;player[i];i++) free(player[i]); 
   free(player);
}


void stop_alarm()
{
  int dummy;
  if (alarm_player_id<=0) {
    gui_print(gui->status,"Alarm is not playing. Can't stop it.\n");
    return ;
  }
  kill(alarm_player_id, SIGKILL);
  proc_end(alarm_player_id, &dummy);
  alarm_player_id= -1;
}


void charge() 
{
   char *dest= get_destination_dir();
   stop_charge= 0;
   charging= 1;
   gui_print(gui->status, "Charging..\n");
   gui_flush();
   if (!dest) {
    gui_print(gui->status, "Charging cancelled.\n  "
           "Please enter a destination directory.");
    charging= 0;
    return ;
   }
   remove_files(dest);
   if (copy_eof_indicator(dest)) return ;
   mp3file_t *files;
   int nfiles;
   int copied;
   char buf[20];
   float *likeliness, *totals;
   get_copy_limit();
   read_database(&files, &nfiles);
   if (!nfiles) {
     gui_print(gui->status,"Charging cancelled. Error reading database.\n");
     free(dest);
     charging= 0;
     return ;
   }
   likeliness= compute_likeliness(files, nfiles);
   totals= compute_likeliness_totals(likeliness, nfiles);
   copied= 0;
   while(1) {
      char *src;
      if (totals[0]<0.1) {
         // unlikely, but may happen if there are a small number of
         // MP3 files and a huge destination directory. In that case,
         // we run out of files to copy and the whole tree of files
         // will have the value 0
         free(likeliness); free(totals);
         likeliness= compute_likeliness(files, nfiles);
         totals= compute_likeliness_totals(likeliness, nfiles);
      }
      src= choose_file_to_copy(files, likeliness, totals, nfiles);
      sprintf(buf,"%06d.mp3", copied);
      int copy_result= copy_file(src, dest, buf);
      // printf("%4d : %s\n", copied, src);
      if (copy_result=='S') break;
      if (copy_result=='O') {
       gui_print(gui->status, "Charging stopped. There was something wrong "
                              "with the destination:\n  %s\n",
                              strerror(copy_errno));
       break; }
      copied++;
      if (copied%config.gui_check==0) {
        gui_flush();
        if (stop_charge) break;
      }
   }
   free(likeliness);
   free(totals);
   write_database(files, nfiles);
   free(dest);
   if (!stop_charge) {
      play_alarm();
      gui_print(gui->status, "Charging complete.\n");
   } else
      gui_print(gui->status,"Charging was stopped manually.\n");
   stop_charge= 0;
   charging= 0;
}



void stop_charging()
{
  stop_charge = 1;
}


int close_main_window()
{
   if (charging) { stop_charging(); return 1; }
   write_config();
   return 0;
}


void show_help_dialog()
{
   gui_help_t *H= gui_c_help(gui->self);
   gui_set_text(H->text, help_about);
   gui_execute(*H);
   gui_close(*H);
}


static void show_settings() {
  int i;
  for(i=0;i<config.N_source_dir;i++)
    add_dir(config.source_dir[i], "    ");
  for(i=0;i<config.N_exclude_dir;i++)
    add_dir(config.exclude_dir[i],"EXC ");
  if (config.G_dest_dir)
    gui_set_text(gui->destdir, config.dest_dir);
  if (config.G_size_limit && config.size_limit<100)
       { config.size_limit= 0; config.G_size_limit= 0; }
  if (config.G_size_limit) {
    gui_set_value(gui->size_limit_on, 1);
    gui_set_value(gui->size_limit, config.size_limit);
  } else {
    gui_set_value(gui->size_limit_on, 0);
    gui_set_value(gui->size_limit, 0);
  }
  if (config.G_eof_indicator) {
    if (!strcmp(config.eof_indicator, "-BLT-"))
      gui_set_value(gui->eof_indicator, 1);
    else {
      gui_set_value(gui->eof_indicator, 2);
      gui_set_text(gui->eof_file, config.eof_indicator);
    }
  } else gui_set_value(gui->eof_indicator, 0);
  if (config.G_alarm_file) {
    if (!strcmp(config.alarm_file, "-BLT-"))
        gui_set_value(gui->alarm, 1);
    else {
      gui_set_value(gui->alarm, 2);
      gui_set_text(gui->alarm_file, config.alarm_file);
    }
  } else gui_set_value(gui->alarm,0);

  if (config.G_alarm_player)
    gui_set_text(gui->alarm_player, config.alarm_player);
  if (config.G_gui_check) {
     if (config.gui_check<0) config.gui_check= 20;
  } else { config.G_gui_check= 1; config.gui_check= 20; }

  gui_set_value(gui->gui_check_period, config.gui_check);
}

static char *config_file_name()
{
  char *home= getenv("HOME");
  static char buf[2000];
  if (!home) return "/etc/cmp3";
  snprintf(buf, 1999, "%s/.cmp3", home);
  buf[1999]= 0;
  return buf;
}


static void remove_from_array
   (char *v, int *size, int *len, char ***A)
{
  char **list= *A;
  int i;
again:
  for(i=0;i<*len;i++) 
    if (!strcmp(list[i],v)) {
      while(i<*len-1) { list[i]= list[i+1]; i++; }
      (*len)--;
      goto again ;
    }
}

static void add_to_array
   (char *v, int *size, int *len, char ***A)
{
  char **list= *A;
  if (*len>=*size) {
     *size+= *size/2;
     list= realloc(list, sizeof(char*)* *size);
     *A= list;
  }
  list[*len]= strdup(v);
  (*len)++;
}


static void add_dir(char *dir,char *kind)
{
  char *V= malloc(strlen(dir)+5);
  sprintf(V, "%s%s", kind, dir);
  gui_append(gui->source_dirs, V, dir);
  free(V);
}


static void unify_array(char **src, int slen, char ***dst, int *dlen)
{
  char **D= malloc((slen+1)*sizeof(char*));
  int i,j;
  for(i=0;i<slen;i++)
   D[i]= src[i];
{

 /* carchive.quick_sort
      type= char*
      var =  D
      size=  slen
      compare=  
    QSRESULT= strcmp(D[QSI], D[QSP]);
 
 */

  int yyqr11[72];
  int yyqr2= -1;
  int yyqr6, QSP, yyqr1, QSI, QSRESULT;
  char* yyqr3;

  yyqr6= 0;
  QSP=  slen - 1;

  if (QSP<=yyqr6) goto yyqr5;

 yyqr4:
 
  yyqr1= yyqr6;

  for(QSI=yyqr6;QSI<QSP;QSI++)
   {
      {  
    QSRESULT= strcmp(D[QSI], D[QSP]);
  }
      if (QSRESULT<=0)
       {
         yyqr3 =  D[QSI];
          D[QSI]=  D[ yyqr1 ];
          D[yyqr1]= yyqr3;
         yyqr1++;
       }
   }
  yyqr3=  D[QSP];
   D[QSP]=  D[yyqr1];
   D[yyqr1]= yyqr3;

  int yyqr10, yyqr9, yyqr8, yyqr7;

  yyqr10 = yyqr6;   yyqr8= yyqr1-1;
  yyqr9 = yyqr1+1; yyqr7 = QSP;

  if (yyqr8-yyqr10 < yyqr7-yyqr9)
  {
    yyqr11[++yyqr2]= yyqr9; yyqr11[++yyqr2]= yyqr7;
    yyqr11[++yyqr2]= yyqr10; yyqr11[++yyqr2]= yyqr8;
  } else {
    yyqr11[++yyqr2]= yyqr10; yyqr11[++yyqr2]= yyqr8;
    yyqr11[++yyqr2]= yyqr9; yyqr11[++yyqr2]= yyqr7;
  }

  while(yyqr2!=-1) {
    QSP= yyqr11[yyqr2--];
    yyqr6= yyqr11[yyqr2--];
    if (yyqr6>=QSP) continue;
    goto yyqr4;
  }
yyqr5:
  yyqr6= QSP;  /* so that gcc doesn't warn us about a label not preceding
                 a statement and such. */

}
  for(i=0,j=0;j<slen;j++)
    if (strcmp(D[j], D[i])) D[++i]= D[j]; 
  *dlen= slen ? (i+1) : 0;
  *dst= D;
}


static int get_sources(char ***src,int *nsrc,char ***ex, int *nex)
{
  char **sources, **excl;
  int nsources, nexcl;
  int i;
  unify_array(config.source_dir, config.N_source_dir, &sources, &nsources);
  unify_array(config.exclude_dir, config.N_exclude_dir, &excl, &nexcl);
  if (nsources==0) {
    gui_print(gui->status,"Scan cancelled: No source directories given\n");
    free(sources); free(excl);
    return 1;
  }
  for(i=0;i<nexcl;i++)
  {
   int j;
   for(j=0;j<nsources;j++)
     if (!strcmp(sources[j],excl[i]))
        { sources[j]= sources[nsources-1]; nsources--; j--; }
  }
  if (nsources==0) {
    gui_print(gui->status,"Scan cancelled: Check config.\n"
          " All source directories are excluded.\n");
    free(sources); free(excl);
    return 1;
  }
  *src= sources; *nsrc= nsources;
  *ex= excl; *nex= nexcl;
  return 0;
}



static void prepare_buffers
   (mp3buffer_t **Rbuffer, dirstack_t **Rdirs,
    char **sources, int nsources)
{
  mp3buffer_t *buffer;
  dirstack_t *dirs;
  int i;
  dirs= dirstack_new(50);
  buffer= mp3buffer_new();
  for(i=0;i<nsources;i++)
    dirstack_push(dirs, strdup(sources[i]));
  free(sources);
  *Rbuffer= buffer; *Rdirs= dirs;
}


static void scan_all 
    (mp3file_t *db, int ndb,
     mp3buffer_t *buffer, dirstack_t *dirs,
     char **excl, int nexcl)
{
  int i;
  gui_print(gui->status,"Scanning directories.\n");
  for(i=0;i<ndb;i++) db[i].aux= 0;
  while(!dirstack_empty(dirs))
  {
    char* name;
    dirstack_pop(dirs, &name);
    for(i=0;i<nexcl;i++)
      if (!strcmp(name,excl[i])) break;
    if (i==nexcl)
      scan_directory(buffer, dirs, name, db, ndb);
    free(name);
  }
  free(excl);
  free(dirs->elts); 
  free(dirs);
}

static void scan_directory
    (mp3buffer_t* buffer, dirstack_t *dirs, char *name, 
     mp3file_t *db, int ndb)
{
   DIR *dir;
   struct dirent *entry;
   static char *nbuf; static int nsize;
   dir= opendir(name);
   if (!dir) return ;
   while((entry=readdir(dir))) {
     if (entry->d_name[0]=='.') continue;
     if (entry->d_type==DT_DIR)
        dirstack_push(dirs, combine(name, entry->d_name));
     else if (entry->d_type==DT_REG) {
        mp3file_t F;
        if (!has_ext(entry->d_name,".mp3")) continue;
        make_path(&nbuf, &nsize, name, entry->d_name);
        if (db && find_file(db, ndb, nbuf)) continue;
        F.name= strdup(nbuf);
        F.hash= 0;
        F.count= F.aux= 0;
        mp3buffer_append(buffer, F);
     }
   }
   closedir(dir);
}


static char *combine(char *dir,char *file)
{
  char *R= malloc(strlen(dir)+strlen(file)+2);
  sprintf(R,"%s/%s",dir,file);
  return R;
}

static int has_ext(char *name, char *ext)
{
  int SL, EL;
  SL= strlen(name); EL= strlen(ext);
  if (SL<=EL) return 0;
  if (!strcasecmp(name+SL-EL,ext)) return 1;
  return 0;
}

static void make_path
   (char **dst, int *dsize,
    char *dir, char *file)
{
  int L= strlen(dir)+strlen(file)+2;
  if (*dsize<L) {
     *dsize= L + L/2;
     if (*dst) *dst= realloc(*dst, *dsize);
     else *dst= malloc(*dsize);
  }
  sprintf(*dst, "%s/%s",dir,file);
}

static int find_file(mp3file_t *db,int ndb, char *name)
{
  int k,R;
again:
  if (ndb<=0) return 0;
  k= ndb/2;
  R= strcmp(name, db[k].name);
  if (R==0) { db[k].aux= 1; return 1; }
  if (R>0) { ndb-= k+1; db+= k+1; goto again; }
  ndb= k;
  goto again;
}

static void flatten_buffer
  (mp3buffer_t *buffer, 
   mp3file_t **Rarray, int *Rn,
   char **Rstrarea)
{
   mp3file_t *array; int n;
   char *strarea;
   int L,i;
   array= mp3buffer_collectd(buffer, &n);
   L= 0;
   for(i=0;i<n;i++)
     L+= strlen(array[i].name)+1;
   strarea= malloc(L+16*n+1);
   L= 0;
   for(i=0;i<n;i++) {
     char *name= array[i].name;
     int SL= strlen(name);
     memcpy(strarea+L, name, SL+1);
     array[i].name= strarea+L;
     L+= SL+1;
     array[i].hash= strarea+L;
     L+= 16;
     free(name);
   }
   *Rarray= array; *Rn= n; *Rstrarea= strarea;
}

static void compute_md5_hash
   (char *path,char *sum)
{
    FILE *f;
    unsigned char *result;
    gcry_error_t ge;
    gcry_md_hd_t gch;

    f= fopen(path,"rb");
    if (!f) { memset(sum, 0,16); return ; }

    ge= gcry_md_open(&gch, GCRY_MD_MD5,0);
    if (ge) {
       static int ep;
       if (!ep) {
         gui_print(gui->status,"gcrypt error: %s\n",
             gcry_strerror(ge));
         ep=1;
       }
       memset(sum, 0,16); fclose(f); return ; 
    }
    char buf[4096];
    int R;
    R= fread(buf, 1, sizeof(buf), f);
    if (R<=0)
      { memset(sum, 0,16); fclose(f); return ; }
    gcry_md_write(gch,buf, R);
    fclose(f);
    result= gcry_md_read(gch, GCRY_MD_MD5);
    if (result) memcpy(sum, result, 16);
    else memset(sum, 0, 16);
    gcry_md_close(gch);
}

static void merge_databases
  (mp3file_t *db, int ndb, mp3buffer_t *buffer,
   mp3file_t **Rdb, int *Rndb,char **Rstrarea)
{
   mp3file_t *newfiles; int n_newfiles;
   mp3file_t *final; int n_final;
   int n_unknown;
   int i,S;

   flatten_buffer(buffer, &newfiles,
                  &n_newfiles, Rstrarea);

   gui_print(gui->status,
        "Computing MD5 hashes for %d files.\n",
         n_newfiles);
   for(i=0;i<n_newfiles;i++)
      compute_md5_hash(newfiles[i].name,
                           newfiles[i].hash);

   if (!ndb)
     { final= *Rdb= newfiles;
       n_final= *Rndb= n_newfiles;
       gui_print(gui->status,"No database was read. "
                              "%d new files.\n",n_newfiles);
       goto sort_return ; }
   int renamed= 0;
   n_unknown= sort_by_aux(db, ndb);
   sort_by_hash(db, n_unknown);
   mp3file_t *found;
   for(i=0;i<n_newfiles;i++)
      if ((found=find_file_by_hash(db, n_unknown,
                 newfiles[i].hash)))
      {
        found->name= newfiles[i].name;
        newfiles[i]= newfiles[n_newfiles-1];
        n_newfiles--; i--;
        renamed++;
      }
   n_unknown= sort_by_aux(db, n_unknown);
   n_final= ndb-n_unknown+n_newfiles;
   gui_print(gui->status,"%d new files, %d renamed files, %d missing.\n"
                         "%d unchanged, total %d files.\n",
          n_newfiles, renamed, n_unknown,
          ndb-n_unknown, n_final);
   final= malloc(sizeof(mp3file_t)*n_final+1);
   S= 0;
   for(i=n_unknown;i<ndb;i++)
     final[S++]= db[i];
   for(i=0;i<n_newfiles;i++)
     final[S++]= newfiles[i];
   *Rdb= final; *Rndb= n_final;
   free(newfiles);
sort_return:
{

 /* carchive.quick_sort
      type= mp3file_t
      var =  final
      size=  n_final
      compare= 
      QSRESULT= strcmp(final[QSI].name, final[QSP].name);
   
 */

  int yyqr11[72];
  int yyqr2= -1;
  int yyqr6, QSP, yyqr1, QSI, QSRESULT;
  mp3file_t yyqr3;

  yyqr6= 0;
  QSP=  n_final - 1;

  if (QSP<=yyqr6) goto yyqr5;

 yyqr4:
 
  yyqr1= yyqr6;

  for(QSI=yyqr6;QSI<QSP;QSI++)
   {
      { 
      QSRESULT= strcmp(final[QSI].name, final[QSP].name);
    }
      if (QSRESULT<=0)
       {
         yyqr3 =  final[QSI];
          final[QSI]=  final[ yyqr1 ];
          final[yyqr1]= yyqr3;
         yyqr1++;
       }
   }
  yyqr3=  final[QSP];
   final[QSP]=  final[yyqr1];
   final[yyqr1]= yyqr3;

  int yyqr10, yyqr9, yyqr8, yyqr7;

  yyqr10 = yyqr6;   yyqr8= yyqr1-1;
  yyqr9 = yyqr1+1; yyqr7 = QSP;

  if (yyqr8-yyqr10 < yyqr7-yyqr9)
  {
    yyqr11[++yyqr2]= yyqr9; yyqr11[++yyqr2]= yyqr7;
    yyqr11[++yyqr2]= yyqr10; yyqr11[++yyqr2]= yyqr8;
  } else {
    yyqr11[++yyqr2]= yyqr10; yyqr11[++yyqr2]= yyqr8;
    yyqr11[++yyqr2]= yyqr9; yyqr11[++yyqr2]= yyqr7;
  }

  while(yyqr2!=-1) {
    QSP= yyqr11[yyqr2--];
    yyqr6= yyqr11[yyqr2--];
    if (yyqr6>=QSP) continue;
    goto yyqr4;
  }
yyqr5:
  yyqr6= QSP;  /* so that gcc doesn't warn us about a label not preceding
                 a statement and such. */

}
}
static int sort_by_aux(mp3file_t *db,int ndb)
{
   int i,j;
   mp3file_t T;
   i=0; j= ndb-1;
   while(j>i) {
     if (db[i].aux==0) { i++; continue; }
     if (db[j].aux==1) { j--; continue; }
     T= db[i]; db[i]= db[j]; db[j]= T;
     i++; j--; 
   }
   for(i=0;i<ndb;i++)
      if (db[i].aux==1) break;
   return i;
}

static void sort_by_hash(mp3file_t *db, int n)
{
{

 /* carchive.quick_sort
      type= mp3file_t
      var =  db
      size=  n
      compare= 
      QSRESULT= memcmp(db[QSI].hash, db[QSP].hash, 16);
    
 */

  int yyqr11[72];
  int yyqr2= -1;
  int yyqr6, QSP, yyqr1, QSI, QSRESULT;
  mp3file_t yyqr3;

  yyqr6= 0;
  QSP=  n - 1;

  if (QSP<=yyqr6) goto yyqr5;

 yyqr4:
 
  yyqr1= yyqr6;

  for(QSI=yyqr6;QSI<QSP;QSI++)
   {
      { 
      QSRESULT= memcmp(db[QSI].hash, db[QSP].hash, 16);
     }
      if (QSRESULT<=0)
       {
         yyqr3 =  db[QSI];
          db[QSI]=  db[ yyqr1 ];
          db[yyqr1]= yyqr3;
         yyqr1++;
       }
   }
  yyqr3=  db[QSP];
   db[QSP]=  db[yyqr1];
   db[yyqr1]= yyqr3;

  int yyqr10, yyqr9, yyqr8, yyqr7;

  yyqr10 = yyqr6;   yyqr8= yyqr1-1;
  yyqr9 = yyqr1+1; yyqr7 = QSP;

  if (yyqr8-yyqr10 < yyqr7-yyqr9)
  {
    yyqr11[++yyqr2]= yyqr9; yyqr11[++yyqr2]= yyqr7;
    yyqr11[++yyqr2]= yyqr10; yyqr11[++yyqr2]= yyqr8;
  } else {
    yyqr11[++yyqr2]= yyqr10; yyqr11[++yyqr2]= yyqr8;
    yyqr11[++yyqr2]= yyqr9; yyqr11[++yyqr2]= yyqr7;
  }

  while(yyqr2!=-1) {
    QSP= yyqr11[yyqr2--];
    yyqr6= yyqr11[yyqr2--];
    if (yyqr6>=QSP) continue;
    goto yyqr4;
  }
yyqr5:
  yyqr6= QSP;  /* so that gcc doesn't warn us about a label not preceding
                 a statement and such. */

}
}
static mp3file_t* find_file_by_hash
    (mp3file_t *db,int n, char *hash)
{
   int i,k,R;
   for(i=0;i<16;i++) if (hash[i]) break;
   if (i==16) return 0; /* erronous hash is
                           never found */
   while(n>0) {
     k= n/2;
     R= memcmp(db[k].hash, hash, 16);
     if (R<0) { db+= k+1; n-= k+1; }
     else if (R>0) { n= k; }
     else { db[k].aux= 1; return db+k; }
   }
   return 0;
}

static void write_database(mp3file_t *files, int nf)
{
   db_header_t header; 
   int str_size, i;
   char *file_name= db_file_name();
   FILE *dbf= fopen(file_name,"wb");
   gui_print(gui->status,"Writing database.\n");
   if (!dbf) {
     gui_print(gui->status, "  can't open %s for writing.\n");
     return ;
   }
   header.nfiles= nf;
   for(str_size=0,i=0;i<nf;i++)
     str_size+= strlen(files[i].name)+1;
   header.size_str= nf*sizeof(mp3file_t)+str_size;
   header.size_full= nf*(sizeof(mp3file_t)+16)+str_size;
   header.magic= CMP3_MAGIC;
   if (write_bytes(dbf, &header, sizeof(header))) return ;
   str_size= 0;
   for(i=0;i<nf;i++)
   {
      mp3file_t F= files[i];
      F.name= (char*)  (str_size+nf*sizeof(mp3file_t));
      F.hash= (char*)  (header.size_str+16*i);
      if (write_bytes(dbf, &F, sizeof(F))) return ;
      str_size+= strlen(files[i].name)+1;
   }
   for(i=0;i<nf;i++)
     if (write_bytes(dbf, files[i].name,
                     strlen(files[i].name)+1)) return ;
   for(i=0;i<nf;i++)
     if (write_bytes(dbf, files[i].hash, 16)) return ;
   fclose(dbf);
   gui_print(gui->status, "  written OK. %d files.\n",nf);
}

static int write_bytes(FILE *f, void *buffer, int len)
{
   int R;
   R= fwrite(buffer, 1, len,f);
   if (R<len) {
     gui_print(gui->status, " error writing file.\n");
     fclose(f);
     return 1;
   }
   return 0;
}

static void read_database(mp3file_t **Rfiles, int *Rnfiles)
{
   char *file_name= db_file_name();
   FILE *dbf;
   db_header_t header;
   int area; mp3file_t *files; int nfiles;
   int i;

   *Rfiles= 0; *Rnfiles= 0;
   gui_print(gui->status, "Reading the database.\n");
   dbf= fopen(file_name,"rb");
   if (!dbf) {
     gui_print(gui->status,"  can't open %s for reading.\n",file_name);
     return ;
   }

   if (read_bytes(dbf, &header, sizeof(header))) return ;
   nfiles= header.nfiles;
   files= malloc(header.size_full);
   area= (int) files;
   
   if (read_bytes(dbf, files, header.size_full)) { free(files); return ; }
   fclose(dbf);
   
   for(i=0;i<nfiles;i++) {
      files[i].name+= area;
      files[i].hash+= area;
   }
  
   gui_print(gui->status, "  read OK. %d files.\n", nfiles);
   *Rfiles= files; *Rnfiles= nfiles;
}

static int read_bytes(FILE *f, void *buffer, int len)
{
   int R;
   R= fread(buffer, 1, len,f);
   if (R<len) {
     gui_print(gui->status, " error reading file.\n");
     fclose(f);
     return 1;
   }
   return 0;
}


static char *db_file_name()
{
  static char buf[2000];
  char *home;
  home= getenv("HOME");
  if (home) { sprintf(buf,"%s/.cmp3-db", home); return buf; }
  return "/var/run/cmp3-db";
}


char **get_player_args()
{
   char *pcmd= gui_get_text(gui->alarm_player);
   char **R;
   char *str;
   int N,i;
   for(i=0;pcmd[i];i++) 
      if ((unsigned char) pcmd[i] <' '
          || pcmd[i]==127)
       pcmd[i]= ' ';
   N=0; str= pcmd;
   R= calloc(strlen(pcmd)+1, sizeof(char*));
   while(1) {
     char *T= strtok(str, " ");
     if (!T) break;
     R[N++]= strdup(T);
     str= 0;
   }
   free(pcmd);
   if (N==0) { free(R); return 0; }
   R[N++]= 0;
   return R;
}



static void play_builtin_alarm()
{
   int fd; char **player; int i;
   char buf[200];
   player= get_player_args();
   if (!player) return ;
   // gui_print(gui->status, "daa dido dido daa da do\n");
   // FIXME: The C library is so broken, I dont even know
   // how to do this properly.
   strcpy(buf,"/tmp/cmp3tmpXXXXXX");
   fd= mkstemp(buf);
   remove(buf);
   if (fd<0) {
     gui_print(gui->status,"can't create temporary file:\n  %s\n"
              "  can't play alarm.\n", strerror(errno));
     goto free_player;
   }
   if (write_ding_wav(fd)) {
        gui_print(gui->status, "Error during write to input stream of "
             "the alarm player:\n  %s\n", strerror(errno));
        close(fd);
        goto free_player;
   }
   alarm_player_id= proc_start(player[0], player,PROC_INP, fd);
   close(fd);
   if (alarm_player_id<=0) {
      gui_print(gui->status, "Could not execute alarm player:\n  %s\n",
         strerror(-alarm_player_id));
   }
free_player:
   for(i=0;player[i];i++) free(player[i]); 
   free(player);
}


char *get_destination_dir()
{
   char *ddir;
   ddir= gui_get_text(gui->destdir);
   trim_string(ddir);
   if (ddir[0]==0) { free(ddir); return 0; }
   return ddir;
}

static int isspc(unsigned int k)
{
  return k<=32 || k==127;
}


static void remove_files(char *dest_dir)
{
   static char *name_buf;
   static int name_buf_size=0;
   struct dirent *entry;
   DIR *dir;
   int fail= 0;
   dir= opendir(dest_dir);
   if (!dir) {
     gui_print(gui->status,"Can't remove files because couldn't open the\n"
                            "destination directory '%s':\n  %s\n"
                            "Trying to copy anyway.\n", dest_dir,
                            strerror(errno));
     return ;
   }
   while((entry=readdir(dir))) {
     if (entry->d_name[0]=='.') continue;
     if (entry->d_type!=DT_REG) continue;
     if (!has_ext(entry->d_name,".mp3")) continue;
     make_path(&name_buf, &name_buf_size, dest_dir, entry->d_name);
     if (remove(name_buf)) fail++;
   }
   closedir(dir);
   if (fail)
     gui_print(gui->status, "File removal failed for %d files.\n", fail);
}

static int copy_file(char *src, char *destdir, char *destname)
{
   int rv= 0;
   int ifd, ofd;
   int L,W,R;
   static char *name_buf; static int name_buf_size;
   static char buf[4096];
   make_path(&name_buf, &name_buf_size,  destdir, destname);
retry_1:
   ifd= open(src, O_RDONLY);
   if (ifd<0) {
     if (errno==EINTR) goto retry_1;
     copy_errno= errno;
     return 'I';
   }
retry_2:
   ofd= open(name_buf, O_WRONLY | O_TRUNC | O_CREAT);
   if (ofd<0) {
      if (errno==EINTR) goto retry_2;
      copy_errno= errno;
      close(ifd);
      if (errno==ENOSPC) return 'S'; 
      return 'O';
   }
   while(1)
   {
     R= read(ifd, buf, sizeof(buf));
     if (R==0) break;
     if (R<0) {
       if (errno==EINTR) continue;
       copy_errno= errno;
       rv= 'I'; break; 
     }
     L= 0;
     while(R) {
       W= write(ofd, buf+L, R);
       if (W<=0) {
         if (errno==EINTR) continue;
         copy_errno= errno;
         if (errno==ENOSPC) rv= 'S'; 
         else rv= 'O';
         break;
       }
       L+= W; R-= W;
       if (copy_limit) {
         copied_bytes += W; 
         if (copied_bytes >= copy_limit) { rv= 'S'; break; }
       }
     }
   }
   close(ifd);
   close(ofd);
   return rv;
}

static float *compute_likeliness(mp3file_t *files, int nfiles)
{
   float *R= malloc(nfiles*sizeof(float)+1);
   int i;
   int maxv= 1;
   for(i=0;i<nfiles;i++)
      maxv= files[i].count > maxv ? files[i].count : maxv;
   for(i=0;i<nfiles;i++) {
      int C= files[i].count;
      C= C? C: 1;
      R[i]= (double) maxv / C;
   }
   return R;
}

static float *compute_likeliness_totals(float *likely, int n)
{
   float *R= malloc(n*sizeof(float)+1);
   int i;
   for(i=0;i<n;i++) R[i]= likely[i];
   for(i=n-1;i>0;i--) R[(i-1)/2]+= R[i];
   return R;
}


static int find_file_by_likeliness
    (float *likeliness, float *tree, int n, double G)
{
   int k=0, ret;
   double lk;
   while(1) {
     int left, right;
     left= 2*k+1; right= 2*k+2;
     if (left>=n) break;
     if (G<=tree[left]) { k=left; continue; }
     G-= tree[left];
     if (G<=likeliness[k]) break;
     G-= likeliness[k];
     if (right>=n) break;
     k= right;
   }
   ret= k;
   lk= likeliness[k];
   likeliness[k]= 0.0;
   while(1) {
     tree[k] -= lk;
     if (!k) break;
     k= (k-1)/2;
   }
   return ret;
}

static char *choose_file_to_copy
     (mp3file_t *files, float *likeliness, float *tree, int nfiles)
{
   double T= (double) rand() / RAND_MAX;
   int sel;
   T *= tree[0];
   sel= find_file_by_likeliness(likeliness, tree, nfiles, T);
   files[sel].count++;
   return files[sel].name;
}

static void get_copy_limit() {
   int limit_flag;
   gui_get_value(gui->size_limit_on,&limit_flag);
   if (limit_flag) {
     int limit;
     gui_get_value(gui->size_limit, &limit);
     copy_limit= limit;
     copy_limit *= 1024 * 1024;
   } else copy_limit= 0;
   copied_bytes= 0;
}

static int copy_eof_indicator(char *dest) 
{
   int V;
   gui_get_value(gui->eof_indicator, &V);
   if (V==0) return 0;
   if (V==1) return copy_ding_mp3(dest);
   char *src= gui_get_text(gui->eof_file);
   if (!src[0]) {
     gui_print(gui->status, "Charging cancelled:\n  Please select a "
                            "file as end-of-files indicator.\n");
     return 1;
   }
   V= copy_file(src, dest, "ZZZ.mp3");
   if (!V) return 0;
   gui_print(gui->status, "Charging cancelled.\n");
   if (V=='O')
       gui_print(gui->status,"  There was something wrong with the"
                             " destination:\n  %s\n", strerror(copy_errno));
   else if (V=='I')
       gui_print(gui->status,"  There was something wrong with the"
                             " source:\n  %s\n", strerror(copy_errno));
   else 
       gui_print(gui->status,"  The destination doesn't have enough space "
                             "even for the end-of-files indicator.\n");
   return 1;
}


static void write_config()
{
   char *fn= config_file_name();
   FILE *f= fopen(fn, "w");
   int i;
   char *sV; int iV;
   if (!f) return ;
   for(i=0;i<config.N_source_dir;i++)
     write_str(f, "source_dir", config.source_dir[i]);
   for(i=0;i<config.N_exclude_dir;i++)
     write_str(f, "exclude_dir", config.exclude_dir[i]);
   gui_get_value(gui->size_limit_on, &iV);
   if (iV) {
     gui_get_value(gui->size_limit, &iV);
     write_int(f,"size_limit", iV);
   }
   sV= gui_get_text(gui->destdir);
   check_and_write(f, "dest_dir", sV);

   gui_get_value(gui->eof_indicator, &iV);
   if (iV==1) 
      write_str(f, "eof_indicator", "-BLT-");
   else if (iV==2) {
      sV= gui_get_text(gui->eof_file);
      check_and_write(f,"eof_indicator", sV);
   }

   gui_get_value(gui->alarm, &iV);
   if (iV==1) 
      write_str(f, "alarm_file", "-BLT-");
   else if (iV==2) {
     sV= gui_get_text(gui->alarm_file);
     check_and_write(f,"alarm_file", sV);
   }

   sV= gui_get_text(gui->alarm_player);
   check_and_write(f, "alarm_player", sV);

   gui_get_value(gui->gui_check_period, &iV);
   write_int(f, "gui_check", iV);
   fclose(f);
}

static void write_int(FILE *f, char *name, int V)
{
  fprintf(f, "%s %d\n",name, V);
}

static void write_str(FILE *f, char *name, char* value)
{
  static char special[256];
  unsigned char *V= (void*) value;
  if (!special[34]) {
    int i;
    char *spec= "\\\"'";
    for(i=1;i<32;i++) special[i]= 1;
    while(*spec) special[*(spec++)]= 1;
    for(i=127;i<256;i++) special[i]= 1;
  }
  fprintf(f, "%s \"", name);
  while(*V) {
    if (special[*V]) fprintf(f,"\\x%02x", *V);
    else fprintf(f,"%c", *V);
    V++;
  }
  fprintf(f,"\"\n");
}

static void check_and_write(FILE *f, char *name, char *value)
{
  trim_string(value);
  if (value[0]) write_str(f, name, value);
}

static void trim_string(char *value)
{
   while(isspc(value[0]))
     memmove(value+0, value+1, strlen(value+1)+1);
   while(strlen(value)>0 && isspc(value[strlen(value)-1]))
     value[strlen(value-1)]= 0;
}


int main(int argc,char **argv)
{
  char *e;
  gui_init(&argc,argv);
  gui= gui_c_main();
  e= cfg_parse_config(config_file_name(), &config);
  if (e) gui_print(gui->status, "error reading config file:\n%s\n", e);
  else gui_print(gui->status, "config file read OK\n");
  show_settings();

  gui_run();
  return 0;
}


